from unittest import TestCase
from k9.helm import *
from k9.core import set_default_namespace, delete_namespace
from k9.cluster_init import *

from k9.pretty_object import PrettyObject

po = PrettyObject()


class TestClusterInit(TestCase):

    def test_jenkins_ready(self):
        """

        """
        pass

    def test_jenkins_logs(self):
        """

        """
        pass

    def test_efk_ready(self):
        """

        """
        pass

    def test_efk_kibana_logs(self):
        """

        """
        pass

    def test_efk_elastic_logs(self):
        """

        """
        pass

    def test_sonarqube_ready(self):
        """

        """
        pass

    def test_sonarqube_logs(self):
        """

        """
        pass

    def test_prometheus_ready(self):
        """

        """
        pass

    def test_prometheus_logs(self):
        """

        """
        pass

    def test_grafana_ready(self):
        """

        """
        pass

    def test_grafana_logs(self):
        """

        """
        pass

    def test_cluster_init_full_np(self):
        """

        """
        pass

    def test_cluster_init_full_cicd(self):
        """

        """
        pass

    def test_jinja_xml(self):
        """

        """
        pass



from kubernetes import client, config
from k9.pretty_object import PrettyObject
from k9.core import get_default_namespace
# po = PrettyObject()
# config.load_kube_config()
#
v1_rbac = client.RbacAuthorizationV1Api()



###########################################################################
# Roles
###########################################################################

def list_roles(namespace: str = None):
    """
    List all cluster roles
    :param namespace: Then namespace to list roles from.  If None, uses the default namespace.
    :return: A list of dictionary items with **name** and **created**.
    """
    if namespace is None:
        namespace = get_default_namespace()

    return [
        {
            'name': role.metadata.name,
            'created': role.metadata.creation_timestamp
        }
        for role in v1_rbac.list_namespaced_role(namespace).items
    ]


def create_role(body: dict, namespace: str = None):
    """
    Create a role from an object defining the role.

    :param body: The role definition object.
    :param namespace: Then namespace to create role in.  If None, uses the default namespace.
    :return: `V1Role <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Role.md>`_

    Example YAML file::

        apiVersion: rbac.authorization.k8s.io/v1
        kind: ClusterRole
        metadata:
          name: ecr-login-role
        rules:
        - apiGroups: [""]
          resources: ["secrets"]
          verbs: ["create", "delete"]
        - apiGroups: [""]
          resources: ["serviceaccounts"]
          verbs: ["get", "patch"]


    Example Call::

        from k9.helper import (
            set_default_namespace,
            create_role,
            read_yaml
        )

        set_default_namespace('my-namespace')

        body = read_yaml('ecr-login-role.yml')
        result = create_role(body)

    """
    if namespace is None:
        namespace = get_default_namespace()

    return v1_rbac.create_namespaced_role(namespace, body)


def delete_role(name: str, namespace: str = None):
    """
    Deletes the specified cluster

    :param name: Name of cluster
    :param namespace: Then namespace to delete role from.  If None, uses the default namespace.
    :return: None if role exists, otherwise return `V1Status <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Status.md>`_
    """
    if namespace is None:
        namespace = get_default_namespace()

    if role_exists(name, namespace):
        return v1_rbac.delete_namespaced_role(name, namespace)
    else:
        return None


def get_role(name: str, namespace: str = None):
    """
    Gets the specified cluster role.

    :param name: Name of cluster role to retrieve.
    :param namespace: Then namespace to get role from.  If None, uses the default namespace.
    :return: `V1Role <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Role.md>`_
    """
    if namespace is None:
        namespace = get_default_namespace()

    return v1_rbac.read_namespaced_role(name, namespace)


def role_exists(name:str, namespace: str = None):
    """
    Checks for the cluster role's existence.

    :param name: Name of cluster role to look for.
    :param namespace: Then namespace to check for role.  If None, uses the default namespace.
    :return: True if specified cluster role exists.
    """
    try:
        if namespace is None:
            namespace = get_default_namespace()

        result = get_role(name, namespace)
        return result.metadata.name == name

    except:
        return False


def create_role_binding(name: str, role: str, sa: str, namespace: str = None):
    """
    Bind the specified role to the specified service account.

    :param name: Name of binding we are creating here
    :param role: The cluster role name to bind with.
    :param sa: The service account to bind this role to.
    :param namespace: The namespace of role and service account.  If namespace is None, then the binding will be performed in the default namespace.
    :return: `V1RoleBinding <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1RoleBinding.md>`_

    Example::

        result = create_role_binding(binding_name, role_name, sa_name)

    """
    if namespace is None:
        namespace = get_default_namespace()

    body =\
        {
            'apiVersion': 'rbac.authorization.k8s.io/v1',
            'kind': 'RoleBinding',
            'metadata': {
                'name': name,
                'namespace': namespace
            },
            'roleRef': {
                'apiGroup': 'rbac.authorization.k8s.io',
                'kind': 'Role',
                'name': role
            },
            'subjects': [{
                'kind': 'ServiceAccount',
                'name': sa,
                'namespace': namespace
            }]
        }
    return v1_rbac.create_namespaced_role_binding(namespace, body)


def get_role_binding(name: str, namespace: str = None):
    """
    Get cluster role binding information

    :param name: Name of cluster role binding
    :param namespace: Then namespace to get role binding from.  If None, uses the default namespace.
    :return: `V1RoleBinding <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1RoleBinding.md>`_
    """
    if namespace is None:
        namespace = get_default_namespace()

    return v1_rbac.read_namespaced_role_binding(name, namespace)


def role_binding_exists(name:str, namespace: str = None):
    """
    Checks for the existence of the cluster role binding.

    :param name: name of cluster role binding.
    :param namespace: Then namespace to check for role binding.  If None, uses the default namespace.
    :return: True if binding exists.
    """
    try:
        if namespace is None:
            namespace = get_default_namespace()

        result = get_role_binding(name, namespace)
        return result.metadata.name == name

    except:
        return False


def delete_role_binding(name: str, namespace: str = None):
    """
    Delete cluster role binding

    :param name: cluster role binding name
    :param namespace: Then namespace to delete role binding from.  If None, uses the default namespace.
    :return: `V1Status <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Status.md>`_
    """
    if namespace is None:
        namespace = get_default_namespace()

    if role_binding_exists(name, namespace):
        return v1_rbac.delete_namespaced_role_binding(name, namespace)
    else:
        return None


###########################################################################
# Cluster Roles
###########################################################################

def list_cluster_roles():
    """
    List all cluster roles
    :return: A list of dictionary items with **name** and **created**.
    """
    return [
        {
            'name': role.metadata.name,
            'created': role.metadata.creation_timestamp
        }
        for role in v1_rbac.list_cluster_role().items
    ]


def create_cluster_role(body: dict):
    """
    Create a cluster role from an object defining the role.

    Example::

            role = {
                'apiVersion': 'rbac.authorization.k8s.io/v1',
                'kind': 'ClusterRole',
                'metadata': {'name': f'{role_name}'},
                'rules': [
                     {
                         'apiGroups': [''],
                         'resources': ['secrets'],
                         'verbs': ['create', 'delete']
                     },
                     {
                         'apiGroups': [''],
                         'resources': ['serviceaccounts'],
                         'verbs': ['get', 'patch']
                     }

                 ]
            }

            result = create_cluster_role(role)

    Result::

        {'aggregation_rule': None,
         'api_version': 'rbac.authorization.k8s.io/v1',
         'kind': 'ClusterRole',
         'metadata': {'annotations': None,
                      'cluster_name': None,
                      'creation_timestamp': datetime.datetime(2019, 10, 16, 17, 33, 28, tzinfo=tzutc()),
                      'deletion_grace_period_seconds': None,
                      'deletion_timestamp': None,
                      'finalizers': None,
                      'generate_name': None,
                      'generation': None,
                      'initializers': None,
                      'labels': None,
                      'managed_fields': None,
                      'name': 'ecr-login-role',
                      'namespace': None,
                      'owner_references': None,
                      'resource_version': '1901293',
                      'self_link': '/apis/rbac.authorization.k8s.io/v1/clusterroles/ecr-login-role',
                      'uid': '10ccdf2b-f03b-11e9-9956-025000000001'},
         'rules': [{'api_groups': [''],
                    'non_resource_ur_ls': None,
                    'resource_names': None,
                    'resources': ['secrets'],
                    'verbs': ['create', 'delete']}]}"

    :param body: The role definition object.
    :return: `V1ClusterRole <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1ClusterRole.md>`_
    """
    return v1_rbac.create_cluster_role(body)


def delete_cluster_role(name: str):
    """
    Deletes the specified cluster

    :param name: Name of cluster
    :return: None if cluster role doesn't exist, otherwise returns `V1Status <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Status.md>`_
    """
    if cluster_role_exists(name):
        return v1_rbac.delete_cluster_role(name)
    else:
        return None


def get_cluster_role(name: str):
    """
    Gets the specified cluster role.

    :param name: Name of cluster role to retrieve.
    :return: `V1ClusterRole <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1ClusterRole.md>`_
    """
    return v1_rbac.read_cluster_role(name)


def cluster_role_exists(name:str):
    """
    Checks for the cluster role's existence.

    :param name: Name of cluster role to look for.
    :return: True if specified cluster role exists.
    """
    try:
        result = get_cluster_role(name)
        return result.metadata.name == name

    except:
        return False


def create_cluster_role_binding(name: str, role: str, sa: str, namespace: str = None):
    """
    Bind the specified role to the specified service account.

    :param name: Name of binding we are creating here.
    :param role: The cluster role name to bind with.
    :param sa: The service account to bind this role to.
    :param namespace: The namespace of the **service account**.  If namespace is None, then the binding will be performed on service account in the default namespace.
    :return: `V1ClusterRoleBinding <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1ClusterRoleBinding.md>`_

    Example::

        result = create_cluster_role_binding(cr_bind_name, cr_name, sa_name)

    """
    if namespace is None:
        namespace = get_default_namespace()

    body =\
        {
            'apiVersion': 'rbac.authorization.k8s.io/v1',
            'kind': 'ClusterRoleBinding',
            'metadata': {'name': name},
            'roleRef': {
                'apiGroup': 'rbac.authorization.k8s.io',
                'kind': 'ClusterRole',
                'name': role
            },
            'subjects': [{
                'kind': 'ServiceAccount',
                'name': sa,
                'namespace': namespace
            }]
        }
    return v1_rbac.create_cluster_role_binding(body)


def get_cluster_role_binding(name: str):
    """
    Get cluster role binding information

    :param name: Name of cluster role binding
    :return: `V1ClusterRoleBinding <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1ClusterRoleBinding.md>`_
    """
    return v1_rbac.read_cluster_role_binding(name)


def cluster_role_binding_exists(name:str):
    """
    Checks for the existence of the cluster role binding.

    :param name: name of cluster role binding.
    :return: True if binding exists.
    """
    try:
        result = get_cluster_role_binding(name)
        return result.metadata.name == name

    except:
        return False


def delete_cluster_role_binding(name: str):
    """
    Delete cluster role binding

    :param name: cluster role binding name
    :return: None if cluster role binding doesn't exist, otherwise returns `V1Status <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Status.md>`_

    """
    if cluster_role_binding_exists(name):
        return v1_rbac.delete_cluster_role_binding(name)
    else:
        return None





from kubernetes import client, config
from k9.pretty_object import PrettyObject
from k9.core import get_default_namespace

po = PrettyObject()
config.load_kube_config()

v1_batch = client.BatchV1beta1Api()


###########################################################################
# Cron Jobs
###########################################################################

def list_cron_jobs(namespace: str = None):
    """
    List all cluster cron jobs
    :param namespace: Then namespace to list cron jobs from.  If None, uses the default namespace.
    :return: A list of dictionary items with **name** and **created**.
    """
    if namespace is None:
        namespace = get_default_namespace()

    return [
        {
            'name': cj.metadata.name,
            'created': cj.metadata.creation_timestamp
        }
        for cj in v1_batch.list_namespaced_cron_job(namespace).items
    ]


def create_cron_job(body: dict, namespace: str = None):
    """
    Create a cron job

    :param body: The cron job definition object.
    :param namespace: Then namespace to create cronjob in.  If None, uses the default namespace.
    :return: `V1beta1CronJob <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1beta1CronJob.md>`_
    """
    if namespace is None:
        namespace = get_default_namespace()

    return v1_batch.create_namespaced_cron_job(namespace, body)


def delete_cron_job(name: str, namespace: str = None):
    """
    Deletes the specified cron job

    :param name: Name of cron job
    :param namespace: Then namespace to delete cron job from.  If None, uses the default namespace.
    :return: None if cron job doesn't exist, `V1Status <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Status.md>`_
    """
    if namespace is None:
        namespace = get_default_namespace()

    if cron_job_exists(name, namespace):
        return v1_batch.delete_namespaced_cron_job(name, namespace)
    else:
        return None


def get_cron_job(name: str, namespace: str = None):
    """
    Gets the specified cron job

    :param name: Name of cron job
    :param namespace: Then namespace to get cron job from.  If None, uses the default namespace.
    :return: `V1beta1CronJob <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1beta1CronJob.md>`_
    """
    if namespace is None:
        namespace = get_default_namespace()

    return v1_batch.read_namespaced_cron_job(name, namespace)


def cron_job_exists(name:str, namespace: str = None):
    """
    Checks for the cron job's existence.

    :param name: Name of cron job to look for.
    :param namespace: Then namespace to check for cron job.  If None, uses the default namespace.
    :return: True if specified cron job exists
    """
    try:
        if namespace is None:
            namespace = get_default_namespace()

        result = get_cron_job(name, namespace)
        return result.metadata.name == name

    except:
        return False



from kubernetes import client, config
from k9.core import *

v1_apps = client.AppsV1Api()

###########################################################################
# Deployments
###########################################################################

def list_deployments(namespace: str = None):
    """
    Lists deployments in a given namespace.

    :param namespace: Namespace you want to search.  If None, the default namespace is used.
    :return: A list of deployment names.
    """
    if namespace is None:
        namespace = get_default_namespace()

    return [
        {
            'name': deployment.metadata.name,
            'ready': f'{deployment.status.ready_replicas}/{deployment.status.replicas}',
            'available': deployment.status.available_replicas,
            'up-to-date': deployment.status.updated_replicas,
            'age': get_age(deployment.metadata.creation_timestamp)
        }
        for deployment in v1_apps.list_namespaced_deployment(namespace).items
    ]


def deployment_exists(name: str, namespace: str = None):
    """
    Returns True if the specified deployment exists.
    
    :param name: The name of the deployment
    :param namespace: The namespace to search in. If none, we use the default namespace.
    :return: True if the deployment exists in the specified namespace (or default if not provided).
    """
    try:
        if namespace is None:
            namespace = get_default_namespace()

        result = get_deployment(name, namespace)
        return result.metadata.name == name

    except:
        return False

def get_deployment(name: str, namespace: str = None):
    """
    Get a specific deployment by name.

    :param name: Name of deployment to retrieve.
    :param namespace:  Namespace to search.  If None, the default namespace is used.

    :return: `V1Deployment <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Deployment.md>`_
    """
    if namespace is None:
        namespace = get_default_namespace()

    return v1_apps.read_namespaced_deployment(name, namespace)


def create_deployment(body: dict, namespace: str = None):
    """
    Create a deployment from a definition file:

    :param body: Deployment Description - https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/AppsV1beta1Deployment.md
    :param namespace: Namespace to create Deployment in.  If None, the default namespace is used.
    :return: `V1Deployment <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Deployment.md>`_

    You'll most likely create a deployment configuration file and import that in as follows:

    Sample YAML file::

        apiVersion: apps/v1
        kind: Deployment
        metadata:
          name: tomcat-dev
        spec:
          replicas: 1
          selector:
            matchLabels:
              app: tomcat
              env: dev
          strategy:
            type: RollingUpdate
            rollingUpdate:
              maxSurge: 1
              maxUnavailable: 1
          minReadySeconds: 5
          template:
            metadata:
              labels:
                app: tomcat
                env: dev
            spec:
              containers:
              - name: tomcat
                image: tomcat:7
                ports:
                  - containerPort: 8080
                env:
                  - name: DS_URL
                    valueFrom:
                      secretKeyRef:
                        name: tomcat-dev
                        key: ds-url
                  - name: DS_USR
                    valueFrom:
                      secretKeyRef:
                        name: tomcat-dev
                        key: username
                  - name: DS_PWD
                    valueFrom:
                      secretKeyRef:
                        name: tomcat-dev
                        key: password

    Example::

        import k9.helper as k9

        body = k9.read_yaml('tomcat-deploy-dev.yml')
        k9.create_deployment(body)


    """
    if namespace is None:
        namespace = get_default_namespace()

    return v1_apps.create_namespaced_deployment(namespace, body)

def delete_deployment(name: str, namespace: str = None):
    """
    Delete the specified deployment.

    :param name: Name of deployment to remove.
    :param namespace: Namespace to remove deployment from.   If None, the default namespace is used.
    :return: None if deployment did not exist, otherwise `V1Status <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Status.md>`_
    """
    if namespace is None:
        namespace = get_default_namespace()

    if deployment_exists(name, namespace):
        return v1_apps.delete_namespaced_deployment(name, namespace)
    else:
        return None


def update_deployment_image(name: str, container: str, image: str, namespace: str = None):
    """
    Updates the specified deployment name with a new image tag for the container.  The container name must
    match the container name specified in the original deployment.

    This will perform a rolling update if  max_unavailable replicas is less than the total number of replicas
    for the cluster.

    :param name: Name of the deployment to update
    :param container: Name of the container within the deployment to update.
    :param image: The image and tag of the image to update the container to.
    :param namespace: The namespace to find the deployment in.  If None, the default namespace is used.
    :return: `V1Deployment <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Deployment.md>`_

    The following example results in a rolling update to to Tomcat 8::

            update_deployment_image(deploy_name, 'tomcat', 'tomcat:8')

    Note that the container name matches the specification of the deployment YAML shown above
    and changes the version of Tomcat from version 7 to version 8.
    """
    if namespace is None:
        namespace = get_default_namespace()

    body = {
        'apiVersion': "apps/v1",
        'kind': "Deployment",
        'metadata': {
            'name': name
        },
        'spec': {
            'template': {
                'spec': {
                    'containers': [
                        {
                            'name': container,
                            'image': image
                        }
                    ]
                }
            }
        }
    }

    return v1_apps.patch_namespaced_deployment(name, namespace, body)

def scale_deployment(name: str, spec: str, namespace:str = None):
    """
    Updates the scaling specification for this deployment.

    :param name: Name of deployment to update.
    :param spec: The updated specification of the scaling parameters.
    :param namespace: The namespace to update the deployment in.  If namespace is None, the default namespace is used.
    :return: `V1Scale <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Scale.md>`_

    Other useful references: https://kubernetes.io/docs/concepts/workloads/controllers/deployment/#scaling-a-deployment

    Example to scale up the deployment to 3 replicas::

            spec = {
                'replicas': 3
            }
            scale_deployment(deploy_name, spec)

    """

    if namespace is None:
        namespace = get_default_namespace()
    body = {
        'apiVersion': "apps/v1",
        'kind': "Deployment",
        'metadata': {
            'name': "tomcat-dev"
        },
        'spec': spec
    }

    return v1_apps.patch_namespaced_deployment_scale(name, namespace, body)



import json

from k9.core import run_command


def create_storage_class(fn: str):
    """
    :param fn: Filename of storage class description
    :return: `subprocess.CompletedProcess <https://docs.python.org/3/library/subprocess.html>`_

    Example::

        from k9.storage import create_storage_class
        result = create_storage_class('storage.yaml')

    Sample YAML file::

        kind: StorageClass
        apiVersion: storage.k8s.io/v1
        metadata:
          name: aws-storage-clas
        provisioner: kubernetes.io/aws-ebs
        parameters:
          type: gp2
          fsType: ext4
        volumeBindingMode: Immediate
        reclaimPolicy: Retain
        allowVolumeExpansion: true

    """

    return run_command('kubectl', 'create', '-f', fn)


def delete_storage_class(name: str):
    """
    Deletes the specfied storage class.

    :param name: Name of storage class
    :return: `subprocess.CompletedProcess <https://docs.python.org/3/library/subprocess.html>`_
    """
    return run_command('kubectl', 'delete', 'storageclass', name)


def list_storage_classes():
    """
    List all storage class

    :return: Returns a list of storage classes.

    Example Output::

        {
            "apiVersion": "v1",
            "items": [
                {
                    "allowVolumeExpansion": true,
                    "apiVersion": "storage.k8s.io/v1",
                    "kind": "StorageClass",
                    "metadata": {
                        "creationTimestamp": "2019-12-20T14:18:10Z",
                        "name": "aws-storage-class",
                        "resourceVersion": "3708823",
                        "selfLink": "/apis/storage.k8s.io/v1/storageclasses/aws-storage-class",
                        "uid": "8d3a7012-2333-11ea-b829-025000000001"
                    },
                    "parameters": {
                        "fsType": "ext4",
                        "type": "gp2"
                    },
                    "provisioner": "kubernetes.io/aws-ebs",
                    "reclaimPolicy": "Retain",
                    "volumeBindingMode": "Immediate"
                },
                {
                    "apiVersion": "storage.k8s.io/v1",
                    "kind": "StorageClass",
                    "metadata": {
                        "annotations": {
                            "storageclass.kubernetes.io/is-default-class": "true"
                        },
                        "creationTimestamp": "2019-10-01T23:34:49Z",
                        "name": "hostpath",
                        "resourceVersion": "527",
                        "selfLink": "/apis/storage.k8s.io/v1/storageclasses/hostpath",
                        "uid": "0f5bccbc-e4a4-11e9-90c3-025000000001"
                    },
                    "provisioner": "docker.io/hostpath",
                    "reclaimPolicy": "Delete",
                    "volumeBindingMode": "Immediate"
                }
            ],
            "kind": "List",
            "metadata": {
                "resourceVersion": "",
                "selfLink": ""
            }
        }
    """

    result = run_command('kubectl', 'get', 'storageclass', '-o', 'json')
    list = json.loads(result.stdout)
    return [
        {
            'name': c['metadata']['name'],
            'allow_volume_expansion': c.get('allowVolumeExpansion'),
            'api_version': c['apiVersion'],
            'provisioner': c['provisioner'],
            'reclaim_policy': c['reclaimPolicy'],
            'volume_binding_mode': c['volumeBindingMode']
        }
        for c in list['items']
    ]


def storage_class_exists(name: str):
    """
    :param name: Storage class name
    :return: True if specified storage class exists.
    """

    list = list_storage_classes()
    found = [
        c['name']
        for c in list
        if c['name'] == name
    ]

    return len(found) > 0




from unittest import TestCase
from k9.core import *
from k9.rbac import *

class TestRbac(TestCase):

    ###########################################################################
    # Roles
    ###########################################################################

    def test_role(self):
        try:
            set_default_namespace('default')

            ############
            # Arrange

            role_name = 'pod-reader-role'
            role = {
                'apiVersion': 'rbac.authorization.k8s.io/v1',
                'kind': 'Role',
                'metadata': {
                    'name': role_name,
                    'namespace': 'default'
                    },
            'rules': [
                    {
                        'apiGroups': [''],
                        'resources': ['pods'],
                        'verbs': ['get', 'watch', 'list']
                    }
                ]
            }

            ############
            # Act
            result = create_role(role)

            ############
            # Assert
            self.assertEqual(role_name, result.metadata.name)

            # test role_exists()
            self.assertTrue(role_exists(role_name))

            # test get_role()
            result = get_role(role_name)
            self.assertEqual(role_name, result.metadata.name)

            # test list_roles()
            found = [
                role
                for role in list_roles()
                if role['name'] == role_name
            ]
            self.assertEqual(1, len(found))

            # test delete_role()
            result = delete_role(role_name)
            self.assertEqual('Success', result.status)

            self.assertFalse(role_exists(role_name))

        finally:
            delete_role(role_name)

    def test_role_exists_fail(self):
        self.assertFalse(role_exists('bogus'))

    def test_role_binding(self):
        try:
            ############
            # Arrange
            set_default_namespace('default')

            sa_name = "xx-pod-reader-sa"
            role_name = "xx-pod-reader-role"
            binding_name = "xx-pod-reader-binding"

            # create the service account
            result = create_service_account(sa_name)
            sa_name = result.metadata.name

            # create role
            body = read_yaml(abs_path('../test/xx-pod-reader-role.yml'))
            result = create_role(body)

            ############
            # Act

            # create cluster role binding
            result = create_role_binding(binding_name, role_name, sa_name)

            ############
            # Assert
            self.assertEqual(binding_name, result.metadata.name)

            # test get_role_binding()
            result = get_role_binding(binding_name)

            # test role_binding_exists()
            self.assertTrue(role_binding_exists(binding_name))

        finally:
            delete_role_binding(binding_name)
            self.assertFalse(role_binding_exists(binding_name))

            delete_role(role_name)
            self.assertFalse(role_exists(role_name))

            delete_service_account(sa_name)
            self.assertFalse(service_account_exists(sa_name))

    def test_delete_bogus_role(self):
        set_default_namespace('default')
        self.assertEqual(None, delete_role('bogus'))

    def test_delete_bogus_role_binding(self):
        set_default_namespace('default')
        self.assertEqual(None, delete_role_binding('bogus'))

    ###########################################################################
    # Cluster Roles
    ###########################################################################

    def test_cluster_role(self):
        try:
            ############
            # Arrange

            role_name = 'ecr-login-role'
            role = {
                'apiVersion': 'rbac.authorization.k8s.io/v1',
                'kind': 'ClusterRole',
                'metadata': {'name': f'{role_name}'},
                'rules': [
                     {
                         'apiGroups': [''],
                         'resources': ['secrets'],
                         'verbs': ['create', 'delete']
                     },
                     {
                         'apiGroups': [''],
                         'resources': ['serviceaccounts'],
                         'verbs': ['get', 'patch']
                     }
                 ]
            }

            ############
            # Act
            result = create_cluster_role(role)

            ############
            # Assert
            self.assertEqual(role_name, result.metadata.name)

            # test cluster_role_exists()
            self.assertTrue(cluster_role_exists(role_name))

            # test list_cluster_roles()
            role_list = [
                role
                for role in list_cluster_roles()
                    if role['name'] == role_name
            ]
            self.assertEqual(role_name, role_list[0]['name'])

            # test delete_cluster_role()
            result = delete_cluster_role(role_name)
            self.assertEqual('Success', result.status)

            self.assertFalse(cluster_role_exists(role_name))

        finally:
            delete_cluster_role(role_name)

    def test_cluster_role_binding(self):
        try:
            ############
            # Arrange
            set_default_namespace('default')

            sa_name = "unit-test-tomcat-sa"
            cr_name = "unit-test-jenkins-access"
            cr_bind_name = "unit-test-jenkins-binding"

            # create the service account
            result = create_service_account(sa_name)
            sa_name = result.metadata.name

            # create cluster role
            body = read_yaml(abs_path('../test/unittest-jenkins-access.yml'))
            result = create_cluster_role(body)

            ############
            # Act

            # create cluster role binding
            result = create_cluster_role_binding(cr_bind_name, cr_name, sa_name)

            ############
            # Assert
            self.assertEqual(cr_bind_name, result.metadata.name)

            # test get_cluster_role_binding()
            result = get_cluster_role_binding(cr_bind_name)

            # test cluster_role_binding_exists()
            self.assertTrue(cluster_role_binding_exists(cr_bind_name))

        finally:
            delete_cluster_role_binding(cr_bind_name)
            self.assertFalse(cluster_role_binding_exists(cr_bind_name))

            delete_cluster_role(cr_name)
            self.assertFalse(cluster_role_exists(cr_name))

            delete_service_account(sa_name)
            self.assertFalse(service_account_exists(sa_name))

    def test_delete_bogus_cluster_role(self):
        self.assertEqual(None, delete_cluster_role('bogus'))

    def test_delete_bogus_cluster_role_binding(self):
        self.assertEqual(None, delete_cluster_role_binding('bogus'))



from kubernetes import client, config
from k9.pretty_object import PrettyObject
from k9.core import *

po = PrettyObject()
config.load_kube_config()

v1_exts = client.ExtensionsV1beta1Api()

###########################################################################
# Ingress
###########################################################################

def list_ingress(namespace: str = None):
    """
    Lists ingresses in a namespace.

    :param namespace: The namespace to search in. If not present, we use the default namespace.
    :return: A list of every ingress in the namespace.

    Example Output::
    
        [
          {
            'name': 'kibana-ing',
            'namespace': 'logging',
            'hosts': [
              'kibana.np.sandbox.simoncomputing.com'
            ],
            'address': [
              '10.10.100.123',
              '10.10.101.145',
              '10.10.102.167'
            ],
            'age': 123456789
          }
        ]
    """
    if namespace is None:
        namespace = get_default_namespace()

    return [
        {
            'name': ing.metadata.name,
            'namespace': ing.metadata.namespace,
            'hosts': [
                tls.hosts
                for tls in ing.spec.tls
            ],
            'address': [
                rule.host
                for rule in ing.spec.rules
            ],
            'age': get_age(ing.metadata.creation_timestamp)
        }
        for ing in v1_exts.list_namespaced_ingress(namespace).items
    ]


def ingress_exists(name: str, namespace: str = None):
    """
    Checks existence of specified ingress.

    :param name: Name of ingress to check.
    :param namespace: Namespace to check, if None, check in default namespace.
    :return: True if specified ingress exists.
    """
    try:
        if namespace is None:
            namespace = get_default_namespace()

        result = get_ingress(name, namespace)
        return result.metadata.name == name

    except:
        return False


def get_ingress(name: str, namespace: str = None):
    """
    Get details of specified ingress.

    :param name: Name of ingress to get.
    :param namespace: Namespace to get ingress from.  If None, get from default namespace.
    :return: `ExtensionsV1beta1Ingress <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/ExtensionsV1beta1Ingress.md>`_
    """
    if namespace is None:
        namespace = get_default_namespace()

    return v1_exts.read_namespaced_ingress(name, namespace)


def create_ingress(body: dict, namespace: str = None):
    """
    Creates an ingress point - which defines

    :param body: Contains Ingress Definition - https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/ExtensionsV1beta1Ingress.md
    :param namespace: Namespace to create ingress in.  If None, use default namespace
    :return: `ExtensionsV1beta1Ingress <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/ExtensionsV1beta1Ingress.md>`_

    Example YAML File::

        apiVersion: extensions/v1beta1
        kind: Ingress
        metadata:
          name: tomcat-ing-dev
        spec:
          rules:
            - host: tomcat.dev.sandbox.simoncomputing.com
              http:
                paths:
                  - path: /
                    backend:
                      serviceName: tomcat-svc-dev
                      servicePort: 8080
          tls:
          - hosts:
            - dev.sandbox.simoncomputing.com

    Example Call:

        from k9.helper import read_yaml, create_ingress

        body = read_yaml('../test/tomcat-ing-dev.yml')
        create_ingress(body)

    """
    if namespace is None:
        namespace = get_default_namespace()

    return v1_exts.create_namespaced_ingress(namespace, body)

def add_ingress_rule(name: str, service_name: str, service_port: int, host: str = None, paths = None, namespace: str = None):
    """
    Adds a rule to an ingress.

    :param name: the name of the ingress to add the rule to
    :param service_name: the service the rule points to
    :param service_port: the port the service is listening on
    :param host: the hostname to use (optional)
    :param paths: list of paths to use (optional). If not supplied, serves on any path ('/')
    :param namespace: the namespace the ingress is in. if not supplied, we use the default namespace
    """

    if namespace is None:
        namespace = get_default_namespace()

    ing = get_ingress(name, namespace) # will throw an exception if not found

    rules = ing.get('spec', {}).get('rules', [])
    new_rule = { "paths": [] }
    if host:
        new_rule['host'] = host
    if paths:
        for p in paths:
            new_rule['paths'].append({
              "path": p,
              "backend": {
                "service_name": service_name,
                "service_port": service_port
              }
            })
    else:
      # add base path
      new_rule['paths'].append({
              "path": "/",
              "backend": {
                "service_name": service_name,
                "service_port": service_port
              }
            })

    rules.append(new_rule)
    ing['spec']['rules'].append(new_rule)
    return v1_exts.patch_namespaced_ingress(name, namespace, ing)


def delete_ingress(name: str, namespace: str = None):
    """
    Deletes specified ingress.

    :param name: Name of ingress
    :param namespace: Namespace to delete from.  If None, remove from default namespace.
    :return: None if ingress doesn't exist, `V1Status <https://github.com/kubernetes-client/python/blob/master/kubernetes/docs/V1Status.md>`_
    """
    if namespace is None:
        namespace = get_default_namespace()

    if ingress_exists(name, namespace):
        return v1_exts.delete_namespaced_ingress(name, namespace)
    else:
        return None



from unittest import TestCase, mock
from k9.exts import *
from k9.apps import *


class TestExts(TestCase):

    ###########################################################################
    # Ingress
    ###########################################################################
    def test_ingress(self):
        try:
            ##################
            # Arrange
            secret_name = "tomcat-dev"
            deploy_name = 'tomcat-dev'
            svc_name = 'tomcat-svc-dev'
            ing_name = 'tomcat-ing-dev'

            # create namespace
            set_default_namespace("ingress-unit-test")
            if not namespace_exists("ingress-unit-test"):
                create_namespace()

            # create secret
            secrets = {
                'ds-url': 'https://some/url',
                'password': 'My1SecretPassword',
                'username': 'postgres'
            }
            create_secret(secret_name, secrets)

            # create deployment
            body = read_yaml(abs_path('../test/tomcat-deploy-dev.yml'))
            create_deployment(body)

            # create service
            body = read_yaml(abs_path('../test/tomcat-svc-dev.yml'))
            create_service(body)

            ##################
            # Act

            # create ingress
            body = read_yaml(abs_path('../test/tomcat-ing-dev.yml'))
            result = create_ingress(body)
            self.assertEqual(ing_name, result.metadata.name)

            ##################
            # Assert

            # test get_ingress()
            result = get_ingress(ing_name)
            self.assertEqual(ing_name, result.metadata.name)

            # test ingress_exists()
            self.assertTrue(ingress_exists(ing_name))

            # test list_ingress()
            found = [
                ing['name']
                for ing in list_ingress()
                if ing['name'] == ing_name
            ]
            self.assertEqual(1, len(found))

        finally:
            delete_ingress(ing_name)
            delete_service(svc_name)
            delete_deployment(deploy_name)
            delete_secret(secret_name)
            delete_namespace()

    def test_ingress_exists_fail(self):
        self.assertFalse(ingress_exists('bogus'))

    def test_delete_bogus_ingress(self):
        set_default_namespace('default')
        self.assertEqual(None, delete_ingress('bogus'))



import json
import os

from jinja2 import Environment, FileSystemLoader
from k9.core import last_word, namespace_exists, get_default_namespace, create_namespace, run_command

def helm_repo_add(repo_name: str, repo_url: str):
    """
    Adds the repository to the current kubernetes environment.

    :param repo_name: Local name of repository
    :param repo_url: URL of repository
    :return: output string on success, exception if failure.
    """
    return run_command('helm', 'repo', 'add', repo_name, repo_url)


def helm_repo_update():
    """
    Update the local copy of the repository with the current
    repository.

    :return: output string on success, exception if failure.
    """
    return run_command('helm', 'repo', 'update')


def helm_repo_ls():
    """
    Lists available repositories.

    :return: Object derived from JSON output, exception on failure.
    """
    result = run_command('helm', 'repo', 'list', '-o', 'json')
    return json.loads(result.stdout)


def helm_repo_remove(repo_name: str):
    """
    Removes a repository.

    :param repo_name:
    :return: output string on success, exception if failure.
    """
    return run_command('helm', 'repo', 'remove', repo_name)


def helm_install(chart_name: str, params, release_name: str = None, values_path: str = None, debug = False, namespace: str = None):
    """
    Runs the Helm installation for the specified chart.   Before the chart
    is installed, the values file which should have the same name as the release_name
    will be evaluated with Jinja, using the parameters to build the new values file.

    The values file is written to the .output/{{release-name}}-values.yaml.

    Helm installation is run with this values.yaml file and on the
    default namespace set using set_default_namespace()

    :param chart_name: Name of Chart.
    :param params: Parameter values in a Dictionary used when pre-processing values.  If it is not a dictionary,
                   the vars() function is run on the object to create a dictionary.
    :param release_name: If None - release_name is defaulted to the last word in chart_name.  For example
           if chart_name is "stable/tomcat", the release_name is "tomcat".   Release name
           is also used to construct the charts values file in the following format:
           ./charts/{{release_name}}-values.yaml
    :param values_path: Override values file location
    :param debug: If True, runs Helm with --debug and --dry-run, which will not actually run
                  and displays the chart values.  Default is False.
    :param namespace: Namespace to install the chart on. If None, we use the default namespace.
    :return: output string on success, exception if failure.

    """
    f = None

    try:
        if namespace is None:
            namespace = get_default_namespace()

        if not namespace_exists(namespace):
            create_namespace(namespace)

        if not isinstance(params, dict):
            params = vars(params)

        # setup release_name
        if release_name is None:
            release_name = last_word(chart_name)

        if values_path:
            print('getting values')
            vp_dir = values_path[0, values_path.rfind('/')]
            env = Environment(loader=FileSystemLoader(vp_dir))
            value_fn = f'{release_name}-values.yaml'
            if not os.path.exists(f'{vp_dir}/{value_fn}'):
                value_fn = f'{release_name}-values.yml'

            if not os.path.exists(f'{vp_dir}/{value_fn}'):
                print(f'values file not found: {vp_dir}/{value_fn}')
                
        else:
            env = Environment(loader=FileSystemLoader('./charts/'))
            value_fn = f'{release_name}-values.yaml'

            if not os.path.exists(f'./charts/{value_fn}'):
                value_fn = f'{release_name}-values.yml'

            if not os.path.exists(f'./charts/{value_fn}'):
                print(f'values file not found: ./charts/{value_fn}')

        template = env.get_template(value_fn)
        template_body = template.render(params)

        if not os.path.exists('./.output'):
            os.mkdir('./.output')

        value_path = f'./.output/{value_fn}'
        f = open(value_path, 'w+')
        f.write(template_body)
        f.close()

        the_call = ['helm', 'install', '-n', namespace, '-f', value_path, release_name, chart_name]
        if debug:
            the_call.extend(['--debug', '--dry-run'])

        return run_command(*the_call)

    finally:
        if f is not None:
            f.close()


def helm_ls(namespace: str = None):
    """
    Lists all current helm installations.

    :param namespace: Namespace to list charts from. If None, we use the default namespace.
    :return: List of all helm installations.  Throws exception on error.

    Sample Output::

        [
            {"name":"tomcat",
            "namespace":"default",
            "revision":"1",
            "updated":"2019-12-10 18:28:43.753814 -0500 EST",
            "status":"deployed",
            "chart":"tomcat-0.4.0",
            "app_version":"7.0"}
        ]

    """

    if namespace is None:
        namespace = get_default_namespace()
    
    result = run_command('helm', '-n', namespace, 'ls', '-o', 'json')
    values = json.loads(result.stdout)
    return values


def helm_exists(release_name: str, namespace: str = None):
    """
    Returns true if the specified release_name exist.

    :param release_name: Name of release.
    :param namespace: Namespace to check for the chart in. If None, we use the default namespace.
    :return: True if found.
    """
    result = helm_ls(namespace)
    found = [
        release
        for release in result
        if release['name'] == release_name
    ]
    return len(found) > 0


def helm_uninstall(release_name: str, namespace: str = None):
    """
    Uninstalls the specified release.

    :param release_name: Name of the release you want to uninstall.
    :param namespace: Namespace to uninstall the chart from. If None, we use the default namespace.
    :return: Returns output value from uninstall process.  Otherwise throws exception.
    """
    if namespace is None:
        namespace = get_default_namespace()
    return run_command('helm', 'uninstall', '-n', namespace, release_name)



import json
import os, time
import requests

from jinja2 import Environment, FileSystemLoader
from k9.helm import helm_install, helm_ls
from k9.core import namespace_exists, set_default_namespace, create_namespace, abs_path, list_pods, wait_for_pod


# util for jinja'ing XML
def write_templated_config(name: str, env, params):
    if not isinstance(params, dict):
        params = vars(params)
    template = env.get_template(name)
    template_body = template.render()
    if not os.path.exists('./.output/config'):
        if not os.path.exists('./.output'):
            os.mkdir('./.output')
        os.mkdir('./.output/config')
    path = f'./.output/config/{name}'
    
    f = open(path, 'w+')
    f.write(template_body)
    f.close()
    return f'./.output/config/{name}'


def install_jenkins(params: dict, namespace = 'cluster-tools'):
    """
    Installs Jenkins to the current kubernetes environment.

    :param params: a dictionary containing the params to insert into the jenkins chart's values.yaml
    :param namespace: the namespace to install jenkins in. defaults to cluster-tools
    :return: True on success, exception if failure.
    """
    set_default_namespace(namespace)
    if not namespace_exists(namespace):
        create_namespace(namespace)

    # get sonarqube token - todo
    sonarqube_token = 'asdf'
    params['sonarqube_token'] = sonarqube_token
    
    ns_ls = helm_ls(namespace)
    # install chart
    helm_install('simoncomputing/jenkins', params, release_name = 'jenkins', values_path = abs_path('yaml/jenkins-values.yml'), namespace = namespace)
    # get pod
    time.sleep(1)
    pods = list_pods(namespace)
    jenkins_pod = False
    for p in pods:
        if p.get('name', '').find('jenkins') != -1:
            jenkins_pod = p.get('name', False)
    # wait for pod to be ready
    if not wait_for_pod(jenkins_pod, namespace, timeout = 300):
        raise('Jenkins pod did not ready within 5 minutes. Please verify that your params are complete and correct.')
    return True
    

def install_efk(params: dict, namespace: str = 'cluster-tools'):
    """
    Installs EFK to the current kubernetes environment.

    :param params: a dictionary containing the params to insert into the efk chart's values.yaml
    :param namespace: the namespace to install efk in. defaults to cluster-tools
    :return: True on success, exception if failure.
    """

    set_default_namespace(namespace)
    if not namespace_exists(namespace):
        create_namespace(namespace)

    ns_ls = helm_ls(namespace)
    # install chart
    helm_install('simoncomputing/efk', params, release_name = 'efk', values_path = abs_path('yaml/efk-values.yml'), namespace = namespace)
    # get pod
    time.sleep(1)
    pods = list_pods(namespace)
    elastic_pod = False
    for p in pods:
        if p.get('name', '').find('elasticsearch') != -1:
            elastic_pod = p.get('name', False)
    # wait for pod to be ready
    if not wait_for_pod(elastic_pod, namespace, timeout = 300):
        raise('Elastic pod did not ready within 5 minutes. Please verify that your params are complete and correct.')

    for p in pods:
        if p.get('name', '').find('kibana') != -1:
            kibana_pod = p.get('name', False)
    # wait for pod to be ready
    if not wait_for_pod(kibana_pod, namespace, timeout = 300):
        raise('Kibana pod did not ready within 5 minutes. Please verify that your params are complete and correct.')

    return True

def install_sonarqube(params: dict, namespace: str = 'cluster_tools'):
    """
    Installs SonarQube to the current kubernetes environment.

    :param params: a dictionary containing the params to insert into the sonarqube chart's values.yaml
    :param namespace: the namespace to install sonarqube in. defaults to cluster-tools
    :return: True on success, exception if failure.
    """
    # install chart
    # get pod
    # wait for pod to be ready
    set_default_namespace(namespace)
    if not namespace_exists(namespace):
        create_namespace(namespace)

    ns_ls = helm_ls(namespace)
    # install chart
    helm_install('simoncomputing/sonarqube', params, release_name = 'efk', values_path = abs_path('yaml/sonarqube-values.yml'), namespace = namespace)
    # get pod
    time.sleep(1)
    pods = list_pods(namespace)
    sonar_pod = False
    for p in pods:
        if p.get('name', '').find('sonar') != -1:
            sonar_pod = p.get('name', False)
    # wait for pod to be ready
    if not wait_for_pod(sonar_pod, namespace, timeout = 300):
        raise('Sonar pod did not ready within 5 minutes. Please verify that your params are complete and correct.')
    return True

def install_prometheus(params: dict, namespace: str = 'cluster_tools'):
    """
    Installs Prometheus to the current kubernetes environment.

    :param params: a dictionary containing the params to insert into the prometheus chart's values.yaml
    :param namespace: the namespace to install prometheus in. defaults to cluster-tools
    :return: True on success, exception if failure.
    """
    # install chart
    # get pod
    # wait for pod to be ready
    set_default_namespace(namespace)
    if not namespace_exists(namespace):
        create_namespace(namespace)

    ns_ls = helm_ls(namespace)
    # install chart
    helm_install('simoncomputing/prometheus', params, release_name = 'prometheus', values_path = abs_path('yaml/prometheus-values.yml'), namespace = namespace)
    # get pod
    time.sleep(1)
    pods = list_pods(namespace)
    prom_pod = False
    for p in pods:
        if p.get('name', '').find('prometheus') != -1:
            prom_pod = p.get('name', False)
    # wait for pod to be ready
    if not wait_for_pod(prom_pod, namespace, timeout = 300):
        raise('Prometheus pod did not ready within 5 minutes. Please verify that your params are complete and correct.')
    return True

def install_grafana(params: dict, namespace: str = 'cluster_tools'):
    """
    Installs grafana to the current kubernetes environment.

    :param params: a dictionary containing the params to insert into the grafana chart's values.yaml
    :param namespace: the namespace to install grafana in. defaults to cluster-tools
    :return: True on success, exception if failure.
    """
    # install chart
    # get pod
    # wait for pod to be ready
    set_default_namespace(namespace)
    if not namespace_exists(namespace):
        create_namespace(namespace)

    ns_ls = helm_ls(namespace)
    # install chart
    helm_install('simoncomputing/grafana', params, release_name = 'grafana', values_path = abs_path('yaml/grafana-values.yml'), namespace = namespace)
    # get pod
    time.sleep(1)
    pods = list_pods(namespace)
    grafana_pod = False
    for p in pods:
        if p.get('name', '').find('grafana') != -1:
            grafana_pod = p.get('name', False)
    # wait for pod to be ready
    if not wait_for_pod(grafana_pod, namespace, timeout = 300):
        raise('Grafana pod did not ready within 5 minutes. Please verify that your params are complete and correct.')
    return True

def install_cluster_tools(params: dict, namespace: str = 'cluster_tools'):
    """
    Installs all cluster tools to the current kubernetes environment. Does not install cicd tools.

    :param params: a dictionary containing the params to insert into the grafana chart's values.yaml
    :param namespace: the namespace to install all cluster tools in. defaults to cluster-tools
    :return: True on success, exception if failure.
    """

    install_prometheus(params, namespace)
    install_grafana(params, namespace)
    install_efk(params, namespace)

    


from unittest import TestCase, mock
from jinja2 import Environment, Template, FileSystemLoader

import io
import os
import yaml

class TestJinja(TestCase):

    def test_template_string(self):
        template = Template('Hello {{ name }}!')
        self.assertEqual('Hello John Doe!', template.render(name='John Doe'))

    def test_template_file(self):
        root = os.path.dirname(os.path.abspath(__file__))
        templates_dir = os.path.join(root, 'charts')
        env = Environment(loader=FileSystemLoader(templates_dir))

        template = env.get_template("jenkins-values.yml")

        result = template.render(domain="fast.sandbox.simoncomputing.com")
        memory_file = io.StringIO(result)

        obj = yaml.safe_load(memory_file)
        self.assertEqual("jenkins.fast.sandbox.simoncomputing.com", obj['jenkins']['clusterZone'])
        self.assertEqual("jenkins.fast.sandbox.simoncomputing.com", obj['jenkins']['master']['ingress']['hostName'])
        self.assertEqual("jenkins.fast.sandbox.simoncomputing.com", obj['jenkins']['master']['ingress']['tls'][0]['hosts'][0])

    def test_template_loop(self):
        workers = [
            {'subnetId': 'subnet-12341234'},
            {'subnetId': 'subnet-34563456'},
            {'subnetId': 'subnet-34563456'},
        ]

        params = {
            'vpc_type': 'prod',
            'color': 'green',
            'workers': workers
        }

        root = os.path.dirname(os.path.abspath(__file__))
        templates_dir = os.path.join(root, 'charts')
        env = Environment(loader=FileSystemLoader(templates_dir))

        template = env.get_template("03-worker.yml")
        value = template.render(params)
        self.assertTrue('Worker0' in value)
        self.assertTrue('Worker1' in value)
        self.assertTrue('Worker2' in value)
        self.assertFalse('Worker3' in value)
